"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.PgMetaschemaPlugin = void 0;

var _taggedTemplateLiteral2 = _interopRequireDefault(require("@babel/runtime/helpers/taggedTemplateLiteral"));

var _graphileUtils = require("graphile-utils");

function _templateObject() {
  var data = (0, _taggedTemplateLiteral2["default"])(["\n        type MetaschemaType {\n          name: String!\n        }\n        type MetaschemaField {\n          name: String!\n          type: MetaschemaType!\n        }\n        type MetaschemaTable {\n          name: String!\n          fields: [MetaschemaField]\n          constraints: [MetaschemaConstraint]\n          foreignKeyConstraints: [MetaschemaForeignKeyConstraint]\n          primaryKeyConstraints: [MetaschemaPrimaryKeyConstraint]\n          uniqueConstraints: [MetaschemaUniqueConstraint]\n          checkConstraints: [MetaschemaCheckConstraint]\n          exclusionConstraints: [MetaschemaExclusionConstraint]\n        }\n        union MetaschemaConstraint =\n            MetaschemaForeignKeyConstraint\n          | MetaschemaUniqueConstraint\n          | MetaschemaPrimaryKeyConstraint\n          | MetaschemaCheckConstraint\n          | MetaschemaExclusionConstraint\n        type MetaschemaForeignKeyConstraint {\n          name: String!\n          fields: [MetaschemaField]\n          refTable: MetaschemaTable\n          refFields: [MetaschemaField]\n        }\n        type MetaschemaUniqueConstraint {\n          name: String!\n          fields: [MetaschemaField]\n        }\n        type MetaschemaPrimaryKeyConstraint {\n          name: String!\n          fields: [MetaschemaField]\n        }\n        type MetaschemaCheckConstraint {\n          name: String!\n          fields: [MetaschemaField]\n        }\n        type MetaschemaExclusionConstraint {\n          name: String!\n          fields: [MetaschemaField]\n        }\n        type Metaschema {\n          tables: [MetaschemaTable]\n        }\n        extend type Query {\n          _meta: Metaschema\n        }\n      "]);

  _templateObject = function _templateObject() {
    return data;
  };

  return data;
}

var PgMetaschemaPlugin = (0, _graphileUtils.makeExtendSchemaPlugin)(function (build, schemaOptions) {
  /** @type {import('graphile-build-pg').PgIntrospectionResultsByKind} */
  var introspection = build.pgIntrospectionResultsByKind;
  /** @type {string[]} */

  var schemas = schemaOptions.pgSchemas;
  return {
    typeDefs: (0, _graphileUtils.gql)(_templateObject()),
    resolvers: {
      // TODO determine why check constraints aren't coming through
      MetaschemaCheckConstraint: {
        /** @param constraint {import('graphile-build-pg').PgConstraint} */
        fields: function fields(constraint) {
          return constraint.keyAttributes;
        }
      },
      MetaschemaExclusionConstraint: {
        /** @param constraint {import('graphile-build-pg').PgConstraint} */
        fields: function fields(constraint) {
          return constraint.keyAttributes;
        }
      },
      MetaschemaUniqueConstraint: {
        /** @param constraint {import('graphile-build-pg').PgConstraint} */
        fields: function fields(constraint) {
          return constraint.keyAttributes;
        }
      },
      MetaschemaPrimaryKeyConstraint: {
        /** @param constraint {import('graphile-build-pg').PgConstraint} */
        fields: function fields(constraint) {
          return constraint.keyAttributes;
        }
      },
      MetaschemaForeignKeyConstraint: {
        /** @param constraint {import('graphile-build-pg').PgConstraint} */
        fields: function fields(constraint) {
          return constraint.keyAttributes;
        },

        /** @param constraint {import('graphile-build-pg').PgConstraint} */
        refTable: function refTable(constraint) {
          return constraint.foreignClass;
        },

        /** @param constraint {import('graphile-build-pg').PgConstraint} */
        refFields: function refFields(constraint) {
          return constraint.foreignKeyAttributes;
        }
      },
      MetaschemaField: {
        /** @param attr {import('graphile-build-pg').PgAttribute} */
        type: function type(attr) {
          return attr.type;
        }
      },
      MetaschemaTable: {
        /** @param table {import('graphile-build-pg').PgClass} */
        fields: function fields(table) {
          return table.attributes.filter(function (attr) {
            if (attr.num < 1) return false; // low-level props

            return true;
          });
        },

        /** @param table {import('graphile-build-pg').PgClass} */
        constraints: function constraints(table) {
          return table.constraints;
        },

        /** @param table {import('graphile-build-pg').PgClass} */
        foreignKeyConstraints: function foreignKeyConstraints(table) {
          return table.constraints.filter(function (c) {
            return c.type === 'f';
          });
        },

        /** @param table {import('graphile-build-pg').PgClass} */
        primaryKeyConstraints: function primaryKeyConstraints(table) {
          return table.constraints.filter(function (c) {
            return c.type === 'p';
          });
        },

        /** @param table {import('graphile-build-pg').PgClass} */
        uniqueConstraints: function uniqueConstraints(table) {
          return table.constraints.filter(function (c) {
            return c.type === 'u';
          });
        },

        /** @param table {import('graphile-build-pg').PgClass} */
        checkConstraints: function checkConstraints(table) {
          return table.constraints.filter(function (c) {
            return c.type === 'c';
          });
        },

        /** @param table {import('graphile-build-pg').PgClass} */
        exclusionConstraints: function exclusionConstraints(table) {
          return table.constraints.filter(function (c) {
            return c.type === 'x';
          });
        }
      },
      MetaschemaConstraint: {
        /** @param obj {import('graphile-build-pg').PgConstraint} */
        __resolveType: function __resolveType(obj) {
          switch (obj.type) {
            case 'p':
              return 'MetaschemaPrimaryKeyConstraint';

            case 'f':
              return 'MetaschemaForeignKeyConstraint';

            case 'c':
              return 'MetaschemaCheckConstraint';

            case 'u':
              return 'MetaschemaUniqueConstraint';

            case 'x':
              return 'MetaschemaExclusionConstraint';
          }
        }
      },
      Metaschema: {
        tables: function tables() {
          return introspection["class"].filter(function (kls) {
            if (!schemas.includes(kls.namespaceName)) return false; // r = ordinary table, i = index, S = sequence, t = TOAST table, v = view, m = materialized view, c = composite type, f = foreign table, p = partitioned table, I = partitioned index

            if (kls.classKind !== 'r') return false;
            return true;
          });
        }
      },
      Query: {
        _meta: function _meta() {
          // just placeholder
          return {};
        }
      }
    }
  };
});
exports.PgMetaschemaPlugin = PgMetaschemaPlugin;
var _default = PgMetaschemaPlugin;
exports["default"] = _default;